/*
    Copyright 2010 Udo Richter.

    Contains code from Klaus Schmidinger's VDR,
    and concepts from Marten Richter

    naludump is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    naludump is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with naludump; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "remux.h"
#include "tools.h"


long long int LastTotalPackets = 0;

cNaluStreamProcessor NaluStreamProcessor;
cPatPmtParser PatPmtParser;

uchar Buffer[65536];

int main(int argc, const char* argv[])
{
    if (argc < 2) {
        printf("Too few arguments");
        exit(1);
    }
    printf("Input file: %s\n", argv[1]);
    int in = open(argv[1], O_RDONLY|O_LARGEFILE);
    if (in == -1) {
        printf("Cannot open %s", argv[1]);
        exit(2);
    }
    int out = 0;
    if (argv[2]) {
        printf("Output file: %s\n", argv[2]);
        out=open(argv[2], O_WRONLY|O_CREAT|O_TRUNC|O_LARGEFILE, DEFFILEMODE);
        if (out == -1) {
            printf("Cannot create %s", argv[2]);
            exit(3);
        }
    }

    NaluStreamProcessor.SetPatPmtParser(&PatPmtParser);


    while (true) {
        int Length = safe_read(in, Buffer, sizeof(Buffer));
        if (Length < 0)
        {
            if (errno == EAGAIN)
                continue;
            printf("Error reading file.\n");
            exit(4);
        }
        if (Length == 0) // eof
            break;

        NaluStreamProcessor.PutBuffer(Buffer, Length);

        while (true)
        {
            int OutLength = 0;
            uchar *OutData = NaluStreamProcessor.GetBuffer(OutLength);
            if (!OutData || OutLength <= 0)
                break;
            write(out, OutData, OutLength);
        }

        long long int TotalPackets = NaluStreamProcessor.GetTotalPackets();
        if (TotalPackets - LastTotalPackets > 10000)
        {
            long long int DroppedPackets = NaluStreamProcessor.GetDroppedPackets();

            printf("\rPackets: %lli Dropped: %lli (%lli%%)",
                   TotalPackets,
                   DroppedPackets,
                   TotalPackets ? DroppedPackets*100/TotalPackets : 0);
            fflush(stdout);
            LastTotalPackets = TotalPackets;
        }
    }

    long long int TotalPackets = NaluStreamProcessor.GetTotalPackets();
    long long int DroppedPackets = NaluStreamProcessor.GetDroppedPackets();
    printf("\rPackets: %lli Dropped: %lli (%lli%%)\n", TotalPackets, DroppedPackets, TotalPackets ? DroppedPackets*100/TotalPackets : 0);
    close(in);
    if (out)
        close(out);
}




